package com.sun.jndi.cosnaming;

import java.security.AccessController;
import javax.naming.NameParser;
import javax.naming.Binding;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import org.omg.CosNaming.NamingContextPackage.NotFound;
import javax.naming.NameNotFoundException;
import javax.naming.Reference;
import javax.naming.InvalidNameException;
import javax.naming.NotContextException;
import javax.naming.CannotProceedException;
import javax.naming.spi.NamingManager;
import javax.naming.Name;
import javax.naming.CompositeName;
import org.omg.CORBA.SystemException;
import java.io.InputStream;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import org.omg.CORBA.INV_OBJREF;
import org.omg.CORBA.COMM_FAILURE;
import javax.naming.CommunicationException;
import org.omg.CORBA.ORBPackage.InvalidName;
import java.util.Iterator;
import java.net.MalformedURLException;
import org.omg.CORBA.BAD_PARAM;
import org.omg.CosNaming.NamingContextHelper;
import javax.naming.ConfigurationException;
import javax.naming.spi.ResolveResult;
import javax.naming.NamingException;
import com.sun.jndi.toolkit.corba.CorbaUtils;
import java.util.Hashtable;
import org.omg.CosNaming.NameComponent;
import org.omg.CosNaming.NamingContext;
import org.omg.CORBA.ORB;
import javax.naming.Context;

public class CNCtx implements Context
{
    private static final boolean debug = false;
    private static ORB _defaultOrb;
    ORB _orb;
    public NamingContext _nc;
    private NameComponent[] _name;
    Hashtable<String, Object> _env;
    static final CNNameParser parser;
    private static final String FED_PROP = "com.sun.jndi.cosnaming.federation";
    boolean federation;
    public static final boolean trustURLCodebase;
    OrbReuseTracker orbTracker;
    int enumCount;
    boolean isCloseCalled;
    
    private static synchronized ORB getDefaultOrb() {
        if (CNCtx._defaultOrb == null) {
            CNCtx._defaultOrb = CorbaUtils.getOrb(null, -1, new Hashtable());
        }
        return CNCtx._defaultOrb;
    }
    
    CNCtx(Hashtable<?, ?> env) throws NamingException {
        this._name = null;
        this.federation = false;
        this.orbTracker = null;
        this.isCloseCalled = false;
        if (env != null) {
            env = (Hashtable)env.clone();
        }
        this._env = env;
        this.federation = "true".equals((env != null) ? env.get("com.sun.jndi.cosnaming.federation") : null);
        this.initOrbAndRootContext(env);
    }
    
    private CNCtx() {
        this._name = null;
        this.federation = false;
        this.orbTracker = null;
        this.isCloseCalled = false;
    }
    
    public static ResolveResult createUsingURL(String s, Hashtable<?, ?> env) throws NamingException {
        CNCtx robj = new CNCtx();
        if (env != null) {
            env = (Hashtable)env.clone();
        }
        robj._env = env;
        return new ResolveResult(robj, CNCtx.parser.parse(robj.initUsingUrl((env != null) ? ((ORB)env.get("java.naming.corba.orb")) : null, s, env)));
    }
    
    CNCtx(ORB orb, OrbReuseTracker orbReuseTracker, NamingContext nc, Hashtable<String, Object> env, NameComponent[] name) throws NamingException {
        this._name = null;
        this.federation = false;
        this.orbTracker = null;
        this.isCloseCalled = false;
        if (orb == null || nc == null) {
            throw new ConfigurationException("Must supply ORB or NamingContext");
        }
        if (orb != null) {
            this._orb = orb;
        }
        else {
            this._orb = getDefaultOrb();
        }
        this._nc = nc;
        this._env = env;
        this._name = name;
        this.federation = "true".equals((env != null) ? env.get("com.sun.jndi.cosnaming.federation") : null);
        return;
    }
    
    NameComponent[] makeFullName(NameComponent[] array) {
        if (this._name == null || this._name.length == 0) {
            return array;
        }
        NameComponent[] array2 = new NameComponent[this._name.length + array.length];
        System.arraycopy(this._name, 0, array2, 0, this._name.length);
        System.arraycopy(array, 0, array2, this._name.length, array.length);
        return array2;
    }
    
    @Override
    public String getNameInNamespace() throws NamingException {
        if (this._name == null || this._name.length == 0) {
            return "";
        }
        return CNNameParser.cosNameToInsString(this._name);
    }
    
    private static boolean isCorbaUrl(String s) {
        return s.startsWith("iiop://") || s.startsWith("iiopname://") || s.startsWith("corbaname:");
    }
    
    private void initOrbAndRootContext(Hashtable<?, ?> hashtable) throws NamingException {
        ORB defaultOrb = null;
        if (defaultOrb == null && hashtable != null) {
            defaultOrb = (ORB)hashtable.get("java.naming.corba.orb");
        }
        if (defaultOrb == null) {
            defaultOrb = getDefaultOrb();
        }
        String s = null;
        if (hashtable != null) {
            s = (String)hashtable.get("java.naming.provider.url");
        }
        if (s != null && !isCorbaUrl(s)) {
            this.setOrbAndRootContext(defaultOrb, this.getStringifiedIor(s));
        }
        else if (s != null) {
            String initUsingUrl = this.initUsingUrl(defaultOrb, s, hashtable);
            if (initUsingUrl.length() > 0) {
                this._name = CNNameParser.nameToCosName(CNCtx.parser.parse(initUsingUrl));
                try {
                    this._nc = NamingContextHelper.narrow(this._nc.resolve(this._name));
                    if (this._nc == null) {
                        throw new ConfigurationException(initUsingUrl + " does not name a NamingContext");
                    }
                }
                catch (BAD_PARAM bad_PARAM) {
                    throw new ConfigurationException(initUsingUrl + " does not name a NamingContext");
                }
                catch (Exception ex) {
                    throw ExceptionMapper.mapException(ex, this, this._name);
                }
            }
        }
        else {
            this.setOrbAndRootContext(defaultOrb, (String)null);
        }
    }
    
    private String initUsingUrl(ORB orb, String s, Hashtable<?, ?> hashtable) throws NamingException {
        if (s.startsWith("iiop://") || s.startsWith("iiopname://")) {
            return this.initUsingIiopUrl(orb, s, hashtable);
        }
        return this.initUsingCorbanameUrl(orb, s, hashtable);
    }
    
    private String initUsingIiopUrl(ORB defaultOrb, String str, Hashtable<?, ?> hashtable) throws NamingException {
        if (defaultOrb == null) {
            defaultOrb = getDefaultOrb();
        }
        try {
            IiopUrl o = new IiopUrl(str);
            NamingException ex = null;
            for (IiopUrl.Address address : o.getAddresses()) {
                try {
                    try {
                        this.setOrbAndRootContext(defaultOrb, defaultOrb.string_to_object("corbaloc:iiop:" + address.host + ":" + address.port + "/NameService"));
                        return o.getStringName();
                    }
                    catch (Exception ex2) {
                        this.setOrbAndRootContext(defaultOrb, (String)null);
                        return o.getStringName();
                    }
                }
                catch (NamingException ex3) {
                    ex = ex3;
                    continue;
                }
                break;
            }
            if (ex != null) {
                throw ex;
            }
            throw new ConfigurationException("Problem with URL: " + str);
        }
        catch (MalformedURLException ex4) {
            throw new ConfigurationException(ex4.getMessage());
        }
    }
    
    private String initUsingCorbanameUrl(ORB defaultOrb, String s, Hashtable<?, ?> hashtable) throws NamingException {
        if (defaultOrb == null) {
            defaultOrb = getDefaultOrb();
        }
        try {
            CorbanameUrl o = new CorbanameUrl(s);
            String location = o.getLocation();
            o.getStringName();
            this.setOrbAndRootContext(defaultOrb, location);
            return o.getStringName();
        }
        catch (MalformedURLException ex) {
            throw new ConfigurationException(ex.getMessage());
        }
    }
    
    private void setOrbAndRootContext(ORB orb, String str) throws NamingException {
        this._orb = orb;
        try {
            org.omg.CORBA.Object object;
            if (str != null) {
                object = this._orb.string_to_object(str);
            }
            else {
                object = this._orb.resolve_initial_references("NameService");
            }
            this._nc = NamingContextHelper.narrow(object);
            if (this._nc == null) {
                if (str != null) {
                    throw new ConfigurationException("Cannot convert IOR to a NamingContext: " + str);
                }
                throw new ConfigurationException("ORB.resolve_initial_references(\"NameService\") does not return a NamingContext");
            }
        }
        catch (InvalidName rootCause) {
            ConfigurationException ex = new ConfigurationException("COS Name Service not registered with ORB under the name 'NameService'");
            ex.setRootCause(rootCause);
            throw ex;
        }
        catch (COMM_FAILURE rootCause2) {
            CommunicationException ex2 = new CommunicationException("Cannot connect to ORB");
            ex2.setRootCause(rootCause2);
            throw ex2;
        }
        catch (BAD_PARAM rootCause3) {
            ConfigurationException ex3 = new ConfigurationException("Invalid URL or IOR: " + str);
            ex3.setRootCause(rootCause3);
            throw ex3;
        }
        catch (INV_OBJREF rootCause4) {
            ConfigurationException ex4 = new ConfigurationException("Invalid object reference: " + str);
            ex4.setRootCause(rootCause4);
            throw ex4;
        }
    }
    
    private void setOrbAndRootContext(ORB orb, org.omg.CORBA.Object obj) throws NamingException {
        this._orb = orb;
        try {
            this._nc = NamingContextHelper.narrow(obj);
            if (this._nc == null) {
                throw new ConfigurationException("Cannot convert object reference to NamingContext: " + obj);
            }
        }
        catch (COMM_FAILURE rootCause) {
            CommunicationException ex = new CommunicationException("Cannot connect to ORB");
            ex.setRootCause(rootCause);
            throw ex;
        }
    }
    
    private String getStringifiedIor(String s) throws NamingException {
        if (s.startsWith("IOR:") || s.startsWith("corbaloc:")) {
            return s;
        }
        InputStream openStream = null;
        try {
            openStream = new URL(s).openStream();
            if (openStream != null) {
                String line;
                while ((line = new BufferedReader(new InputStreamReader(openStream, "8859_1")).readLine()) != null) {
                    if (!line.startsWith("IOR:")) {
                        continue;
                    }
                    try {
                        if (openStream != null) {
                            openStream.close();
                        }
                    }
                    catch (IOException rootCause) {
                        ConfigurationException ex = new ConfigurationException("Invalid URL: " + s);
                        ex.setRootCause(rootCause);
                        throw ex;
                    }
                    return line;
                }
            }
            try {
                if (openStream != null) {
                    openStream.close();
                }
            }
            catch (IOException rootCause2) {
                ConfigurationException o = new ConfigurationException("Invalid URL: " + s);
                o.setRootCause(rootCause2);
                throw o;
            }
        }
        catch (IOException rootCause3) {
            ConfigurationException o2 = new ConfigurationException("Invalid URL: " + s);
            o2.setRootCause(rootCause3);
            throw o2;
        }
        finally {
            try {
                if (openStream != null) {
                    openStream.close();
                }
            }
            catch (IOException rootCause4) {
                ConfigurationException o3 = new ConfigurationException("Invalid URL: " + s);
                o3.setRootCause(rootCause4);
                throw o3;
            }
        }
        throw new ConfigurationException(s + " does not contain an IOR");
    }
    
    Object callResolve(NameComponent[] array) throws NamingException {
        try {
            org.omg.CORBA.Object resolve = this._nc.resolve(array);
            try {
                NamingContext narrow = NamingContextHelper.narrow(resolve);
                if (narrow != null) {
                    return new CNCtx(this._orb, this.orbTracker, narrow, this._env, this.makeFullName(array));
                }
                return resolve;
            }
            catch (SystemException ex) {
                return resolve;
            }
        }
        catch (Exception ex2) {
            throw ExceptionMapper.mapException(ex2, this, array);
        }
    }
    
    @Override
    public Object lookup(String n) throws NamingException {
        return this.lookup(new CompositeName(n));
    }
    
    @Override
    public Object lookup(Name name) throws NamingException {
        if (this._nc == null) {
            throw new ConfigurationException("Context does not have a corresponding NamingContext");
        }
        if (name.size() == 0) {
            return this;
        }
        NameComponent[] nameToCosName = CNNameParser.nameToCosName(name);
        Object refInfo;
        try {
            refInfo = this.callResolve(nameToCosName);
            try {
                if (CorbaUtils.isObjectFactoryTrusted(refInfo)) {
                    refInfo = NamingManager.getObjectInstance(refInfo, name, this, this._env);
                }
            }
            catch (NamingException ex) {
                throw ex;
            }
            catch (Exception rootCause) {
                NamingException ex2 = new NamingException("problem generating object using object factory");
                ex2.setRootCause(rootCause);
                throw ex2;
            }
        }
        catch (CannotProceedException ex3) {
            return getContinuationContext(ex3).lookup(ex3.getRemainingName());
        }
        return refInfo;
    }
    
    private void callBindOrRebind(NameComponent[] array, Name name, Object obj, boolean b) throws NamingException {
        if (this._nc == null) {
            throw new ConfigurationException("Context does not have a corresponding NamingContext");
        }
        try {
            obj = NamingManager.getStateToBind(obj, name, this, this._env);
            if ((obj instanceof CNCtx)) {
                obj = ((CNCtx)obj)._nc;
            }
            if ((obj instanceof NamingContext)) {
                NamingContext narrow = NamingContextHelper.narrow((org.omg.CORBA.Object)obj);
                if (b) {
                    this._nc.rebind_context(array, narrow);
                }
                else {
                    this._nc.bind_context(array, narrow);
                }
            }
            else if ((obj instanceof org.omg.CORBA.Object)) {
                if (b) {
                    this._nc.rebind(array, (org.omg.CORBA.Object)obj);
                }
                else {
                    this._nc.bind(array, (org.omg.CORBA.Object)obj);
                }
            }
            else {
                throw new IllegalArgumentException("Only instances of org.omg.CORBA.Object can be bound");
            }
        }
        catch (BAD_PARAM rootCause) {
            NotContextException ex = new NotContextException(name.toString());
            ex.setRootCause(rootCause);
            throw ex;
        }
        catch (Exception ex2) {
            throw ExceptionMapper.mapException(ex2, this, array);
        }
        return;
    }
    
    @Override
    public void bind(Name name, Object o) throws NamingException {
        if (name.size() == 0) {
            throw new InvalidNameException("Name is empty");
        }
        NameComponent[] nameToCosName = CNNameParser.nameToCosName(name);
        try {
            this.callBindOrRebind(nameToCosName, name, o, false);
        }
        catch (CannotProceedException ex) {
            getContinuationContext(ex).bind(ex.getRemainingName(), o);
        }
        return;
    }
    
    private static Context getContinuationContext(CannotProceedException cpe) throws NamingException {
        try {
            return NamingManager.getContinuationContext(cpe);
        }
        catch (CannotProceedException ex) {
            Object resolvedObj = ex.getResolvedObj();
            if (resolvedObj instanceof Reference && ((Reference)resolvedObj).get("nns").getContent() instanceof Context) {
                NameNotFoundException ex2 = new NameNotFoundException("No object reference bound for specified name");
                ex2.setRootCause(cpe.getRootCause());
                ex2.setRemainingName(cpe.getRemainingName());
                throw ex2;
            }
            throw ex;
        }
    }
    
    @Override
    public void bind(String n, Object o) throws NamingException {
        this.bind(new CompositeName(n), o);
    }
    
    @Override
    public void rebind(Name name, Object o) throws NamingException {
        if (name.size() == 0) {
            throw new InvalidNameException("Name is empty");
        }
        NameComponent[] nameToCosName = CNNameParser.nameToCosName(name);
        try {
            this.callBindOrRebind(nameToCosName, name, o, true);
        }
        catch (CannotProceedException ex) {
            getContinuationContext(ex).rebind(ex.getRemainingName(), o);
        }
        return;
    }
    
    @Override
    public void rebind(String n, Object o) throws NamingException {
        this.rebind(new CompositeName(n), o);
    }
    
    private void callUnbind(NameComponent[] array) throws NamingException {
        if (this._nc == null) {
            throw new ConfigurationException("Context does not have a corresponding NamingContext");
        }
        try {
            this._nc.unbind(array);
        }
        catch (NotFound notFound) {
            if (!this.leafNotFound(notFound, array[array.length - 1])) {
                throw ExceptionMapper.mapException(notFound, this, array);
            }
        }
        catch (Exception ex) {
            throw ExceptionMapper.mapException(ex, this, array);
        }
        return;
    }
    
    private boolean leafNotFound(NotFound notFound, NameComponent nameComponent) {
        NameComponent nameComponent2;
        return notFound.why.value() == 0 && notFound.rest_of_name.length == 1 && (nameComponent2 = notFound.rest_of_name[0]).id.equals(nameComponent.id) && (nameComponent2.kind == nameComponent.kind || (nameComponent2.kind != null && nameComponent2.kind.equals(nameComponent.kind)));
    }
    
    @Override
    public void unbind(String n) throws NamingException {
        this.unbind(new CompositeName(n));
    }
    
    @Override
    public void unbind(Name name) throws NamingException {
        if (name.size() == 0) {
            throw new InvalidNameException("Name is empty");
        }
        NameComponent[] nameToCosName = CNNameParser.nameToCosName(name);
        try {
            this.callUnbind(nameToCosName);
        }
        catch (CannotProceedException ex) {
            getContinuationContext(ex).unbind(ex.getRemainingName());
        }
        return;
    }
    
    @Override
    public void rename(String n, String n2) throws NamingException {
        this.rename(new CompositeName(n), new CompositeName(n2));
    }
    
    @Override
    public void rename(Name name, Name name2) throws NamingException {
        if (this._nc == null) {
            throw new ConfigurationException("Context does not have a corresponding NamingContext");
        }
        if (name.size() == 0 || name2.size() == 0) {
            throw new InvalidNameException("One or both names empty");
        }
        this.bind(name2, this.lookup(name));
        this.unbind(name);
        return;
    }
    
    @Override
    public NamingEnumeration<NameClassPair> list(String n) throws NamingException {
        return this.list(new CompositeName(n));
    }
    
    @Override
    public NamingEnumeration<NameClassPair> list(Name name) throws NamingException {
        return this.listBindings(name);
    }
    
    @Override
    public NamingEnumeration<Binding> listBindings(String n) throws NamingException {
        return this.listBindings(new CompositeName(n));
    }
    
    @Override
    public NamingEnumeration<Binding> listBindings(Name name) throws NamingException {
        if (this._nc == null) {
            throw new ConfigurationException("Context does not have a corresponding NamingContext");
        }
        if (name.size() > 0) {
            try {
                Object lookup = this.lookup(name);
                if ((lookup instanceof CNCtx)) {
                    return new CNBindingEnumeration((CNCtx)lookup, true, this._env);
                }
                throw new NotContextException(name.toString());
            }
            catch (NamingException ex) {
                throw ex;
            }
            catch (BAD_PARAM rootCause) {
                NotContextException ex2 = new NotContextException(name.toString());
                ex2.setRootCause(rootCause);
                throw ex2;
            }
        }
        else {
            return new CNBindingEnumeration(this, false, this._env);
        }
    }
    
    private void callDestroy(NamingContext namingContext) throws NamingException {
        if (this._nc == null) {
            throw new ConfigurationException("Context does not have a corresponding NamingContext");
        }
        try {
            namingContext.destroy();
        }
        catch (Exception ex) {
            throw ExceptionMapper.mapException(ex, this, (NameComponent[])null);
        }
        return;
    }
    
    @Override
    public void destroySubcontext(String n) throws NamingException {
        this.destroySubcontext(new CompositeName(n));
    }
    
    @Override
    public void destroySubcontext(Name name) throws NamingException {
        if (this._nc == null) {
            throw new ConfigurationException("Context does not have a corresponding NamingContext");
        }
        NamingContext namingContext = this._nc;
        NameComponent[] nameToCosName = CNNameParser.nameToCosName(name);
        if (name.size() > 0) {
            try {
                CNCtx cnCtx = (CNCtx)(Context)this.callResolve(nameToCosName);
                namingContext = cnCtx._nc;
                cnCtx.close();
            }
            catch (ClassCastException ex) {
                throw new NotContextException(name.toString());
            }
            catch (CannotProceedException ex2) {
                getContinuationContext(ex2).destroySubcontext(ex2.getRemainingName());
                return;
            }
            catch (NameNotFoundException ex3) {
                if (ex3.getRootCause() instanceof NotFound && this.leafNotFound((NotFound)ex3.getRootCause(), nameToCosName[nameToCosName.length - 1])) {
                    return;
                }
                throw ex3;
            }
            catch (NamingException ex4) {
                throw ex4;
            }
        }
        this.callDestroy(namingContext);
        this.callUnbind(nameToCosName);
        return;
    }
    
    private Context callBindNewContext(NameComponent[] array) throws NamingException {
        if (this._nc == null) {
            throw new ConfigurationException("Context does not have a corresponding NamingContext");
        }
        try {
            return new CNCtx(this._orb, this.orbTracker, this._nc.bind_new_context(array), this._env, this.makeFullName(array));
        }
        catch (Exception ex) {
            throw ExceptionMapper.mapException(ex, this, array);
        }
    }
    
    @Override
    public Context createSubcontext(String n) throws NamingException {
        return this.createSubcontext(new CompositeName(n));
    }
    
    @Override
    public Context createSubcontext(Name name) throws NamingException {
        if (name.size() == 0) {
            throw new InvalidNameException("Name is empty");
        }
        NameComponent[] nameToCosName = CNNameParser.nameToCosName(name);
        try {
            return this.callBindNewContext(nameToCosName);
        }
        catch (CannotProceedException ex) {
            return getContinuationContext(ex).createSubcontext(ex.getRemainingName());
        }
    }
    
    @Override
    public Object lookupLink(String n) throws NamingException {
        return this.lookupLink(new CompositeName(n));
    }
    
    @Override
    public Object lookupLink(Name name) throws NamingException {
        return this.lookup(name);
    }
    
    @Override
    public NameParser getNameParser(String s) throws NamingException {
        return CNCtx.parser;
    }
    
    @Override
    public NameParser getNameParser(Name name) throws NamingException {
        return CNCtx.parser;
    }
    
    @Override
    public Hashtable<String, Object> getEnvironment() throws NamingException {
        if (this._env == null) {
            return new Hashtable(5, 0.75f);
        }
        return (Hashtable)this._env.clone();
    }
    
    @Override
    public String composeName(String n, String n2) throws NamingException {
        return this.composeName(new CompositeName(n), new CompositeName(n2)).toString();
    }
    
    @Override
    public Name composeName(Name name, Name name2) throws NamingException {
        return ((Name)name2.clone()).addAll(name);
    }
    
    @Override
    public Object addToEnvironment(String key, Object value) throws NamingException {
        if (this._env == null) {
            this._env = new Hashtable(7, 0.75f);
        }
        else {
            this._env = (Hashtable)this._env.clone();
        }
        return this._env.put(key, value);
    }
    
    @Override
    public Object removeFromEnvironment(String s) throws NamingException {
        if (this._env != null && this._env.get(s) != null) {
            this._env = (Hashtable)this._env.clone();
            return this._env.remove(s);
        }
        return null;
    }
    
    public synchronized void incEnumCount() {
        ++this.enumCount;
    }
    
    public synchronized void decEnumCount() throws NamingException {
        --this.enumCount;
        if (this.enumCount == 0 && this.isCloseCalled) {
            this.close();
        }
    }
    
    @Override
    public synchronized void close() throws NamingException {
        if (this.enumCount > 0) {
            this.isCloseCalled = true;
            return;
        }
        return;
    }
    
    @Override
    protected void finalize() {
        try {
            this.close();
        }
        catch (NamingException ex) {}
    }
    
    static {
        parser = new CNNameParser();
        trustURLCodebase = "true".equalsIgnoreCase((String)AccessController.doPrivileged(()/*  */ -> System.getProperty("com.sun.jndi.cosnaming.object.trustURLCodebase", "false")));
    }
}
